@@ -329,7 +329,8 @@ static int handle_buffered_output(AVAudioResampleContext *avr,
 
 int attribute_align_arg avresample_convert(AVAudioResampleContext *avr,
                                            uint8_t **output, int out_plane_size,
-                                           int out_samples, uint8_t **input,
+                                           int out_samples,
+                                           uint8_t * const *input,
                                            int in_plane_size, int in_samples)
 {
     AudioData input_buffer;
